@@ -24,7 +24,11 @@ int TestVRMLNormals(int argc, char* argv[])
 
   char* fname = vtkTestUtilities::ExpandDataFileName(argc, argv, "Data/WineGlass.wrl");
   importer->SetFileName(fname);
-  importer->Read();
+  if (!importer->Update())
+  {
+    std::cerr << "ERROR: Importer failed to update\n";
+    return EXIT_FAILURE;
+  }
 
   delete[] fname;
 
